@@ -259,41 +259,42 @@ sub send {
     my ($self, $content) = @_;
 
     Claude::Agent::Error->throw(message => 'Not connected') unless $self->_connected;
-    Claude::Agent::Error->throw(message => 'No active query') unless $self->_query;
-    # Note: We intentionally do NOT pre-check is_finished here due to race conditions.
-    # The try/catch block below handles the case where query finishes between any
-    # check and the actual write operation, providing robust error handling.
 
-    # Attempt to send the message, catching write errors gracefully
-    require Try::Tiny;
-    my $write_error;
-    my $original_exception;
-    Try::Tiny::try {
-        $self->_query->send_user_message($content);
-    }
-    Try::Tiny::catch {
-        $original_exception = $_;
-        # Stringify for logging but preserve original for re-throw
-        $write_error = ref($_) ? "$_" : $_;
-        $log->debug(sprintf("Client::send write error: %s", $write_error));
-    };
+    # Get session_id from previous query or stored value
+    my $session_id = $self->_session_id // ($self->_query ? $self->_query->session_id : undef);
+    Claude::Agent::Error->throw(message => 'No session_id available for send') unless $session_id;
 
-    # If write failed and query is now finished, throw appropriate error
-    if ($write_error) {
-        if ($self->_query->is_finished) {
-            Claude::Agent::Error->throw(message => 'Query finished during send');
-        }
-        # Re-throw original exception if it's an object to preserve stack trace and type
-        # Otherwise create a new error with the message
-        if (ref($original_exception) && $original_exception->can('throw')) {
-            $original_exception->throw();
-        }
-        Claude::Agent::Error->throw(
-            message => "Send failed: $write_error",
-            ($original_exception ? (cause => $original_exception) : ()),
-        );
+    # Cleanup previous query if it exists
+    if ($self->_query) {
+        $self->_query->cleanup();
     }
 
+    $log->debug(sprintf("Client: Sending follow-up, resuming session id=%s", $session_id));
+
+    # Create new options with resume
+    my $opts = $self->options;
+    my $resume_opts = Claude::Agent::Options->new(
+        ($opts->has_allowed_tools ? (allowed_tools => $opts->allowed_tools) : ()),
+        ($opts->has_disallowed_tools ? (disallowed_tools => $opts->disallowed_tools) : ()),
+        ($opts->has_model ? (model => $opts->model) : ()),
+        ($opts->has_permission_mode ? (permission_mode => $opts->permission_mode) : ()),
+        ($opts->has_mcp_servers ? (mcp_servers => $opts->mcp_servers) : ()),
+        ($opts->has_hooks ? (hooks => $opts->hooks) : ()),
+        ($opts->has_agents ? (agents => $opts->agents) : ()),
+        ($opts->has_max_turns ? (max_turns => $opts->max_turns) : ()),
+        ($opts->has_system_prompt ? (system_prompt => $opts->system_prompt) : ()),
+        resume => $session_id,
+    );
+
+    # Create new query with resume
+    $self->_query(
+        Claude::Agent::Query->new(
+            prompt  => $content,
+            options => $resume_opts,
+            ($self->has_loop ? (loop => $self->loop) : ()),
+        )
+    );
+
     return $self;
 }
 
